#include <QAbstractSocket>

#include "jsondemo.h"


#define HARDWARE_PORT_IN		8012
#define HARDWARE_PORT_OUT		8011
#define SOFTWARE_PORT_IN		8094
#define SOFTWARE_PORT_OUT		8093



JSONDemo::JSONDemo(QWidget *parent)
: QDialog(parent)
, m_iPortIn(HARDWARE_PORT_IN)
, m_iPortOut(HARDWARE_PORT_OUT)
, m_socket(NULL)
{
	setMinimumSize(500, 300);

	QRegExp rx = QRegExp("(([0-2]?[0-9][0-9]?)(\\.|$)){4}");
	QRegExpValidator ipv4Validator(rx);

	QLabel* pLabelIpLoc = new QLabel("Local IP :", this);

	m_pEditIpLoc = new QLineEdit("127.0.0.1", this);
	m_pEditIpLoc->setAlignment(Qt::AlignCenter);
	m_pEditIpLoc->setValidator(&ipv4Validator);

	QLabel* pLabelIpDest = new QLabel("Dest. IP :", this);

	m_pEditIpDest = new QLineEdit("127.0.0.1", this);
	m_pEditIpDest->setAlignment(Qt::AlignCenter);
	m_pEditIpDest->setValidator(&ipv4Validator);

	QPushButton* pButConnectToHw = new QPushButton("Connect to Hardware", this);
	connect(pButConnectToHw, SIGNAL(pressed()), this, SLOT(connectToHw()));

	QPushButton* pButConnectToSw = new QPushButton("Connect to Software", this);
	connect(pButConnectToSw, SIGNAL(pressed()), this, SLOT(connectToSw()));

	QPushButton* pButGetSerial = new QPushButton("1. Get Serial", this);
	connect(pButGetSerial, SIGNAL(pressed()), this, SLOT(getSerial()));

	QPushButton* pButGetState = new QPushButton("2. Get State", this);
	connect(pButGetState, SIGNAL(pressed()), this, SLOT(getState()));

	QPushButton* pButPlayScene = new QPushButton("3. Play Scene #1", this);
	connect(pButPlayScene, SIGNAL(pressed()), this, SLOT(playScene()));

	m_pLabelMessage = new QLabel("No JSON Message received...", this);
	m_pLabelMessage->setAlignment(Qt::AlignCenter);
	m_pLabelMessage->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

	QScrollArea* pScrollArea = new QScrollArea(this);
	pScrollArea->setWidget(m_pLabelMessage);
	pScrollArea->setWidgetResizable(true);

	QHBoxLayout* pHLayoutIpLoc = new QHBoxLayout;
	pHLayoutIpLoc->addWidget(pLabelIpLoc);
	pHLayoutIpLoc->addWidget(m_pEditIpLoc);

	QHBoxLayout* pHLayoutIpDest = new QHBoxLayout;
	pHLayoutIpDest->addWidget(pLabelIpDest);
	pHLayoutIpDest->addWidget(m_pEditIpDest);

	QHBoxLayout* pHLayoutConnect = new QHBoxLayout;
	pHLayoutConnect->addWidget(pButConnectToHw);
	pHLayoutConnect->addWidget(pButConnectToSw);
	
	QHBoxLayout* pHLayoutButtons = new QHBoxLayout;
	pHLayoutButtons->addWidget(pButGetSerial);
	pHLayoutButtons->addWidget(pButGetState);
	pHLayoutButtons->addWidget(pButPlayScene);

	QVBoxLayout* pMainLayout = new QVBoxLayout;
	pMainLayout->addLayout(pHLayoutIpLoc,   0);
	pMainLayout->addLayout(pHLayoutConnect, 0);
	pMainLayout->addLayout(pHLayoutIpDest,  0);
	pMainLayout->addLayout(pHLayoutButtons, 0);
	pMainLayout->addWidget(pScrollArea,     1);
	setLayout(pMainLayout);
}

JSONDemo::~JSONDemo()
{
}

void JSONDemo::connectToHw()
{
	m_iPortIn  = HARDWARE_PORT_IN;
	m_iPortOut = HARDWARE_PORT_OUT;

	if (!connectSocket(m_pEditIpLoc->text()))
		QMessageBox::information(this, "Error", "Cannot connect to the specified IP");
}
	
void JSONDemo::connectToSw()
{
	m_iPortIn  = SOFTWARE_PORT_IN;
	m_iPortOut = SOFTWARE_PORT_OUT;

	if (!connectSocket(m_pEditIpLoc->text()))
		QMessageBox::information(this, "Error", "Cannot connect to the specified IP");
}

bool JSONDemo::connectSocket(const QString& strIp)
{
	if (m_socket)
	{
		m_socket->abort();
	}
	else
	{
		m_socket = new QUdpSocket(this);
		connect(m_socket, SIGNAL(readyRead()), this, SLOT(readPacket()));
	}

	if (strIp.isEmpty())
	{
		return m_socket->bind(m_iPortIn, QAbstractSocket::ShareAddress | QAbstractSocket::ReuseAddressHint);
	}

	return m_socket->bind(QHostAddress(strIp), m_iPortIn, QAbstractSocket::ShareAddress | QAbstractSocket::ReuseAddressHint);
}

void JSONDemo::getSerial()
{
	if (m_socket) 
	{
		JSONNode serial_node;
		serial_node.push_back(JSONNode("typ", (int)getserial));
		std::string str = serial_node.write_formatted();

		QByteArray data(str.c_str());

		m_socket->writeDatagram(data, QHostAddress(m_pEditIpDest->text()), m_iPortOut);
	}
}

void JSONDemo::getState()
{
	if (m_socket) 
	{
		m_dataBuffer.clear();

		JSONNode state_node;
		state_node.push_back(JSONNode("typ", (int)getstate));
		std::string str = state_node.write_formatted();

		QByteArray data(str.c_str());

		m_socket->writeDatagram(data, QHostAddress(m_pEditIpDest->text()), m_iPortOut);
	}
}

void JSONDemo::playScene()
{
	static bool bState = true;

	if (m_socket) 
	{
		JSONNode scn_live;
		JSONNode scn_live_content;
		scn_live_content.set_name("content");

		scn_live_content.push_back(JSONNode("id", 1));
		scn_live_content.push_back(JSONNode("a", 1));
		scn_live_content.push_back(JSONNode("s", bState));

		scn_live.push_back(JSONNode("typ", (int)scn));
		scn_live.push_back(scn_live_content);

		std::string str = scn_live.write_formatted();

		QByteArray data(str.c_str());

		m_socket->writeDatagram(data, QHostAddress(m_pEditIpDest->text()), m_iPortOut);

		bState = !bState;
	}
}

void JSONDemo::readPacket()
{
	while (m_socket->hasPendingDatagrams())
	{
		QByteArray datagram;
		QHostAddress sender;
		quint16 senderPort;

		int nBytesReceived = m_socket->pendingDatagramSize();

		if (nBytesReceived > 0)
		{
			datagram.resize(nBytesReceived);
			m_socket->readDatagram(datagram.data(), nBytesReceived, &sender, &senderPort);

			analysePacket(datagram, sender.toString());
		}
		else
		{
			m_socket->flush();
			return;
		}
	}
}

void JSONDemo::analysePacket(const QByteArray& data, const QString& strIp)
{
	JSONNode node(JSON_NULL);
	QString strJSON(data.constData());

	if (strJSON.endsWith("!#("))
	{
		m_dataBuffer.append(data);
		m_dataBuffer.remove(m_dataBuffer.size() - 3, 3);
		strJSON = m_dataBuffer.constData();

		if (strJSON.count("{") != strJSON.count("}"))
			return;

		std::string json = m_dataBuffer.constData();
		node = libjson::parse(json);
		
		m_dataBuffer.clear();
	}
	else
	{
		std::string json = data.constData();
		node = libjson::parse(json);
	}

	/***/

	if (node.type() == JSON_NULL)
		return;

	spMessageType eType = none;
	JSONNode::const_iterator i = node.begin();

    while (i != node.end())
	{ 
        std::string str = i->name();
 
        if (str == "typ")
		{
			eType = (spMessageType)i->as_int();

			switch (eType)
			{
				case alive:
					/* Device or Server is still alive */
				break;

				case limit:
					/* Number of users max exceeded */
				break;

				default:
					m_pLabelMessage->setText(strJSON);
				break;
			}
		}
		else if (str == "content")
		{
			/*if (i->type() == JSON_NODE)
			{*/

			switch (eType)
			{
				case serial:		readSerial(*i, strIp);		break;
				case state:			readState(*i, strIp);		break;
				case scn:			readScene(*i);				break;
				case trg:			readTrigger(*i);			break;
				case clr:			readColor(*i);				break;
				case clrlist:		readColorArray(*i);			break;
				case cmd:			readCommand(*i);			break;
				case cmdlist:		readCommandArray(*i);		break;
				case dmx:			readDmx(*i);				break;
				case univ:			readUniverse(*i);			break;
				case scnstate:		readSceneState(*i, strIp);	break;
				case timeline:		readTimeline(*i);			break;
				case clockstate:	readClock(*i);				break;
				case nodestate:		readNode(*i);				break;
				default:			return;
			}

			return;

			/*}*/
		}

        ++i;
    }
}

void JSONDemo::readSerial(const JSONNode& node, const QString& strIp)
{
	JSONNode::const_iterator i = node.begin();

    while (i != node.end())
	{
        std::string str = i->name();
 
        if (str == "serial")
		{
			QString strSerial = i->as_string().c_str();
		}
		else if (str == "name")
		{
			QString strName = i->as_string().c_str();
		}
		else if (str == "msstate")
		{
			bool bMaster = i->as_bool();
		}
		else if (str == "signal")
		{
			bool bSignal = i->as_bool();
		}

        ++i;
    }
}

void JSONDemo::readState(const JSONNode& node, const QString& strIp)
{
	spMessageType eType = none;
	JSONNode::const_iterator i = node.begin();

    while (i != node.end())
	{ 
        std::string str = i->name();
 
		if (str == "scn")
		{
			JSONNode::const_iterator j = i->begin();

			while (j != i->end())
			{
				str = j->name();
				if (str == "content")
					readSceneArray(*j);

				++j;
			}
		}
		else if (str == "trg")
		{
			JSONNode::const_iterator j = i->begin();

			while (j != i->end())
			{
				str = j->name();
				if (str == "content")
					readTriggerArray(*j);

				++j;
			}
		}
		else if (str == "clr")
		{
			JSONNode::const_iterator j = i->begin();

			while (j != i->end())
			{
				str = j->name();
				if (str == "content")
					readColor(*j);

				++j;
			}
		}
		else if (str == "clrlist")
		{
			JSONNode::const_iterator j = i->begin();

			while (j != i->end())
			{
				str = j->name();
				if (str == "content")
					readColorArray(*j);

				++j;
			}
		}
		else if (str == "cmd")
		{
			JSONNode::const_iterator j = i->begin();

			while (j != i->end())
			{
				str = j->name();
				if (str == "content")
					readCommand(*j);

				++j;
			}
		}
		else if (str == "cmdlist")
		{
			JSONNode::const_iterator j = i->begin();

			while (j != i->end())
			{
				str = j->name();
				if (str == "content")
					readCommandArray(*j);

				++j;
			}
		}
       
        ++i;
    }
}

void JSONDemo::readSceneState(const JSONNode& node, const QString& strIp)
{
	spMessageType eType = none;
	JSONNode::const_iterator i = node.begin();

    while (i != node.end())
	{ 
        std::string str = i->name();
 
		if (str == "scn")
		{
			JSONNode::const_iterator j = i->begin();

			while (j != i->end())
			{
				str = j->name();
				if (str == "content")
					readSceneArray(*j);

				++j;
			}
		}
		else if (str == "trg")
		{
			JSONNode::const_iterator j = i->begin();

			while (j != i->end())
			{
				str = j->name();
				if (str == "content")
					readTriggerArray(*j);

				++j;
			}
		}
       
        ++i;
    }
}

void JSONDemo::readSceneArray(const JSONNode& node)
{
	if (node.type() != JSON_ARRAY)
		return;
	
	JSONNode::const_iterator i = node.begin();

    while (i != node.end())
	{
        readScene(*i);
        ++i;
    }		
}

void JSONDemo::readTriggerArray(const JSONNode& node)
{
	if (node.type() != JSON_ARRAY)
		return;

	JSONNode::const_iterator i = node.begin();

    while (i != node.end())
	{
        readTrigger(*i);
        ++i;
    }
}

void JSONDemo::readScene(const JSONNode& node)
{
	m_scene.n_  = "";
    m_scene.id_ = 0;
    m_scene.a_  = 0;
    m_scene.l_  = 0;
    m_scene.s_  = true;

	JSONNode::const_iterator i = node.begin();

    while (i != node.end())
	{ 
        std::string str = i->name();
 
        if		(str == "n")	m_scene.n_  = i->as_string();
        else if (str == "id")	m_scene.id_ = i->as_int();
        else if (str == "a")	m_scene.a_  = i->as_int();
        else if (str == "l")	m_scene.l_  = i->as_int();
		else if (str == "s")	m_scene.s_  = i->as_bool();
       
        ++i;
    }
}

void JSONDemo::readTrigger(const JSONNode& node)
{
    m_trigger.n_  = "";
    m_trigger.id_ = 0;
	m_trigger.a_  = 0;
    m_trigger.s_  = true;

	JSONNode::const_iterator i = node.begin();

    while (i != node.end())
	{
        std::string str = i->name();
 
		if		(str == "n")	m_trigger.n_  = i->as_string();
        else if (str == "id")	m_trigger.id_ = i->as_int();
		else if (str == "a")	m_trigger.a_  = i->as_int();
		else if (str == "s")	m_trigger.s_  = i->as_bool();
       
        ++i;
    }
}

void JSONDemo::readColorArray(const JSONNode& node)
{
	if (node.type() != JSON_ARRAY)
		return;

	JSONNode::const_iterator i = node.begin();

    while (i != node.end())
	{
        readColor(*i);
        ++i;
    }
}

void JSONDemo::readColor(const JSONNode& node)
{
	m_color.rgbw_     = false;
    m_color.s_        = false;
    m_color.color_.r_ = 255;
    m_color.color_.g_ = 255;
    m_color.color_.b_ = 255;
    m_color.color_.w_ = 255;
    m_color.a_        = 0;

	JSONNode::const_iterator i = node.begin();

    while (i != node.end())
	{
		std::string str = i->name();
 
		if (str == "color")// && i->type() == JSON_NODE)
		{
			JSONNode clrNode = node.at("color");
			JSONNode::const_iterator j = clrNode.begin();

			while (j != clrNode.end())
			{
				str = j->name();
			 
				if		(str == "r")	m_color.color_.r_ = j->as_int();
				else if (str == "g")	m_color.color_.g_ = j->as_int();
				else if (str == "b")	m_color.color_.b_ = j->as_int();
				else if (str == "w")	m_color.color_.w_ = j->as_int();
			    
				++j;
			}
		}
		else if (str == "rgbw")
		{
			m_color.rgbw_ = i->as_bool();
		}
		else if (str == "s")
		{
			m_color.s_ = i->as_bool();
		}
		else if (str == "a")
		{
			m_color.a_ = i->as_int();
		}

		++i;
    }   
}

void JSONDemo::readCommandArray(const JSONNode& node)
{
	if (node.type() != JSON_ARRAY)
		return;

	JSONNode::const_iterator i = node.begin();

    while (i != node.end())
	{
        readCommand(*i);
        ++i;
    }
}

void JSONDemo::readCommand(const JSONNode& node)
{
	m_cmd.blackout_ = false;
    m_cmd.next_     = false;
    m_cmd.prev_     = false;
    m_cmd.white_    = false;
    m_cmd.hold_     = false;
    m_cmd.dim_      = 50;
    m_cmd.speed_    = 50;
    m_cmd.a_        = 0;

	JSONNode::const_iterator i = node.begin();

    while (i != node.end())
	{
        std::string str = i->name();
 
		if		(str == "bo")		m_cmd.blackout_ = i->as_bool();
		else if (str == "next")		m_cmd.next_     = i->as_bool();
		else if (str == "prev")		m_cmd.prev_     = i->as_bool();
		else if (str == "white")	m_cmd.white_    = i->as_bool();
		else if (str == "hold")		m_cmd.hold_     = i->as_bool();
		else if (str == "dim")		m_cmd.dim_      = i->as_int();
		else if (str == "speed")	m_cmd.speed_    = i->as_int();
        else if (str == "a")		m_cmd.a_        = i->as_int();

        ++i;
    }
}

void JSONDemo::readTimeline(const JSONNode& node)
{
	m_timeline.play_  = false;
    m_timeline.pause_ = false;
    m_timeline.stop_  = false;
    m_timeline.go_    = false;
    m_timeline.prev_  = false;
    m_timeline.next_  = false;

	JSONNode::const_iterator i = node.begin();

    while (i != node.end())
	{
        std::string str = i->name();
		
		if		(str == "play")		m_timeline.play_  = i->as_bool();
		else if (str == "pause")	m_timeline.pause_ = i->as_bool();
		else if (str == "stop")		m_timeline.stop_  = i->as_bool();
		else if (str == "go")		m_timeline.go_    = i->as_bool();
		else if (str == "prev")		m_timeline.prev_  = i->as_bool();
		else if (str == "next")		m_timeline.next_  = i->as_bool();

        ++i;
    }
}

void JSONDemo::readClock(const JSONNode& node)
{
	m_clock.hour_     = 0;
    m_clock.min_      = 0;
    m_clock.sec_      = 0;
    m_clock.year_     = 2020;
    m_clock.month_    = 1;
    m_clock.day_      = 1;

	JSONNode::const_iterator i = node.begin();

    while (i != node.end())
	{
        std::string str = i->name();
 
		if		(str == "hour")		m_clock.hour_  = i->as_int();
		else if (str == "minute")	m_clock.min_   = i->as_int();
		else if (str == "second")	m_clock.sec_   = i->as_int();
		else if (str == "year")		m_clock.year_  = i->as_int();
		else if (str == "month")	m_clock.month_ = i->as_int();
		else if (str == "day")		m_clock.day_   = i->as_int();

        ++i;
    }
}

void JSONDemo::readNode(const JSONNode& node)
{
	m_node.sn_     = "";
    m_node.ln_     = "";
    m_node.ip_     = "192.168.0.1";
    m_node.mask_   = "255.255.255.0";
    m_node.subnet_ = 0;
    m_node.univ_   = 0;
    m_node.cfg_    = 0;

	JSONNode::const_iterator i = node.begin();

    while (i != node.end())
	{
        std::string str = i->name();

		if		(str == "shortname")	m_node.sn_     = i->as_string();
		else if (str == "longname")		m_node.ln_     = i->as_string();
		else if (str == "ip")			m_node.ip_     = i->as_string();
		else if (str == "mask")			m_node.mask_   = i->as_string();
		else if (str == "subnet")		m_node.subnet_ = i->as_int();
		else if (str == "universe")		m_node.univ_   = i->as_int();
		else if (str == "config")		m_node.cfg_    = i->as_int();

        ++i;
    }
}

void JSONDemo::readDmx(const JSONNode& node)
{
	m_chan.c_ = 0;
	m_chan.u_ = 0;
	m_chan.l_ = 0;
	m_chan.s_ = false;

	JSONNode::const_iterator i = node.begin();

    while (i != node.end())
	{
        std::string str = i->name();
 
        if		(str == "chan")		m_chan.c_ = i->as_int();
		else if (str == "univ")		m_chan.u_ = i->as_int();
		else if (str == "level")	m_chan.l_ = i->as_int();
		else if (str == "s")		m_chan.s_ = i->as_bool();

        ++i;
    }
}

void JSONDemo::readUniverse(const JSONNode& node)
{
	int iUniverse = -1;

	JSONNode::const_iterator i = node.begin();

    while (i != node.end())
	{
        std::string str = i->name();
 
        if (str == "universe")
		{
			iUniverse = i->as_int();
		}
		else if (str == "channels")
		{
			if (iUniverse >= 0 && iUniverse < 4)
				readUnivChannels(iUniverse, *i);
		}

        ++i;
    }
}

void JSONDemo::readUnivChannels(int iUniverse, const JSONNode& node)
{
	if (node.type() != JSON_ARRAY)
		return;

	JSONNode::const_iterator i = node.begin();

    while (i != node.end())
	{
        readUnivChannel(iUniverse, *i);
        ++i;
    }
}

void JSONDemo::readUnivChannel(int iUniverse, const JSONNode& node)
{
	int index = -1;
	JSONNode::const_iterator i = node.begin();

	while (i != node.end())
	{
		std::string str = i->name();

		if (str == "c")
		{
			index = i->as_int();
			/*if (index < 0 || index >= 512)
				return;*/
		}
		else if (str == "l")
		{
			if (index >= 0 && index < 512)
				int level = i->as_int();
		}
		else if (str == "s")
		{
			if (index >= 0 && index < 512)
				bool state = i->as_bool();
		}
		
		++i;
	}
}