#ifndef JSONDEMO_H
#define JSONDEMO_H

#include <QtWidgets>
#include <QtWidgets/QDialog>
#include <QtNetwork/QUdpSocket>

#include "libjson/libjson.h"
#include "spdata.h"


class JSONDemo : public QDialog
{
	Q_OBJECT

public:
	JSONDemo(QWidget *parent = 0);
	~JSONDemo();

private slots:
	void connectToHw();
	void connectToSw();

	void getSerial();
	void getState();
	void playScene();

	void readPacket();

private:
	int					m_iPortIn;
	int					m_iPortOut;

	QUdpSocket*			m_socket;
	QByteArray			m_dataBuffer;

	Scene				m_scene;
	Trigger				m_trigger;
	Color				m_color;
	Command				m_cmd;
	DmxChan				m_chan;
	Timeline			m_timeline;
	Clock				m_clock;
	Node				m_node;

	QLineEdit*			m_pEditIpLoc;
	QLineEdit*			m_pEditIpDest;
	QLabel*				m_pLabelMessage;

	bool connectSocket(const QString& strIp);
	void analysePacket(const QByteArray& data, const QString& strIp);

	void readSerial(const JSONNode& node, const QString& strIp);
	void readState(const JSONNode& node, const QString& strIp);
	void readSceneState(const JSONNode& node, const QString& strIp);
	void readSceneArray(const JSONNode& node);
	void readTriggerArray(const JSONNode& node);
	void readScene(const JSONNode& node);
	void readTrigger(const JSONNode& node);
	void readColorArray(const JSONNode& node);
	void readColor(const JSONNode& node);
	void readCommandArray(const JSONNode& node);
	void readCommand(const JSONNode& node);
	void readTimeline(const JSONNode& node);
	void readClock(const JSONNode& node);
	void readNode(const JSONNode& node);
	void readDmx(const JSONNode& node);
	void readUniverse(const JSONNode& node);
	void readUnivChannels(int iUniverse, const JSONNode& node);
	void readUnivChannel(int iUniverse, const JSONNode& node);
};

#endif // JSONDEMO_H
